<?php

namespace Transferito\Models\Core;

use Transferito\Models\Core\Config;

class Api {

	private $uploadUrl;
	private $savedAPI;
	private $planDetail;
	private $createUrl;
	private $statusUrl;

	/**
	 * Keys
	 */
	private $publicKey;
	private $secretKey;

	public function __construct()
	{
		$settings = get_option("transferito_settings_option");
		$this->uploadUrl = Config::getEndpoint('import');
		$this->savedAPI = Config::getEndpoint('information');
		$this->planDetail = Config::getEndpoint('detail');
		$this->createUrl = Config::getEndpoint('create');
		$this->statusUrl = Config::getEndpoint('status');

		/**
		 * API Keys
		 */
		$this->publicKey = sanitize_text_field($settings["public_transferito_key"]);
		$this->secretKey = sanitize_text_field($settings["secret_transferito_key"]);

	}

	public function create($transferDetail)
	{
		return $this->post($this->createUrl, $transferDetail);
	}

	public function getStatus($token)
	{
		return $this->get("{$this->statusUrl}/{$token}");
	}

	public function planInformation()
	{
		return $this->post($this->planDetail, array(), true);
	}

	public function files(array $files, $transferToken)
	{
		return $this->post($this->uploadUrl, array(
			"transferKey"   => sanitize_text_field($transferToken),
			"database"      => $files["database"],
			"codebase"      => $files["codebase"]
		), true);
	}

	public function getSavedDetail()
	{
		$result = array(
			'savedFTP'  => [],
			'savedDBS'  => []
		);

		/**
		 * Call api - to get saved details
		 */
		$savedDetail = $this->post($this->savedAPI, array());

		/**
		 * If the response is success
		 * Then create the result array
		 */
		if ($savedDetail['code'] === 200) {
			$response = $savedDetail['message'];
			$result = array(
				'savedFTP'  => isset($response->newFTP) ? $response->newFTP : [],
				'savedDBs'  => isset($response->database) ? $response->database : []
			);
		}

		return $result;
	}

	private function getRequestHeaders()
	{
		return array(
			"pKey"      => $this->publicKey,
			"sKey"      => $this->secretKey,
			"platform"  => 'wp_plugin',
			"token"     => wp_create_nonce('wp_plugin')
		);
	}

	private function post($url, $body, $returnResultProperty = false)
	{
		/**
		 * Call the endpoint
		 */
		$response = wp_remote_post($url, array(
			'method'    => 'POST',
			'timeout'   => 600,
			'body'      => $body,
			'headers'   => $this->getRequestHeaders()
		));
		return $this->handleResponse($response, $returnResultProperty);
	}

	private function get($url, $returnResultProperty = false)
	{
		$response = wp_remote_get($url, array(
			'headers'   => $this->getRequestHeaders()
		));
		return $this->handleResponse($response, $returnResultProperty);
	}

	private function handleResponse($response, $returnResultProperty = false)
	{
		/**
		 *
		 */
		$httpCode = wp_remote_retrieve_response_code($response);

		/**
		 * Check on the result of the transaction
		 */
		if (is_wp_error($response)) {
			$responseMessage = $response->get_error_message();
		} else {
			$result = wp_remote_retrieve_body($response);
			$jsonResponse = json_decode($result);
			$responseMessage = ($returnResultProperty) ? $jsonResponse->result : $jsonResponse;
		}

		// close the session
		return array(
			'message'   => $responseMessage,
			'code'      => $httpCode
		);

	}

}