<?php

namespace Transferito\Models\Transfer;


class Archive {

	/**
	 * @param \ZipArchive $archive
	 * @param $directoryPath
	 * @param string $baseFolder
	 * @param $selectedFoldersEnabled
	 * @param $folderPathArray
	 */
    public function createDirectoryArchive(
	    $excludeTransferitoPlugin,
    	\ZipArchive $archive,
	    $directoryPath,
	    $baseFolder = "",
		$selectedFoldersEnabled = false,
		$folderPathArray = []
		)
    {
        $fileList = new \RecursiveIteratorIterator(
            new \RecursiveDirectoryIterator($directoryPath),
            \RecursiveIteratorIterator::LEAVES_ONLY
        );

        foreach ($fileList as $name => $file)
        {

            // Skip directories (they would be added automatically)
            if (!$file->isDir())
            {
                // Get real and relative path for current file
                $filePath = $file->getRealPath();
                $fileArray = explode("/",$filePath);

                $relativePath = $fileArray[count($fileArray) - 1];
                $relativePath = substr($filePath, strlen($directoryPath));

	            $excludeTransferitoUploadFolder = (strpos($filePath, TRANSFERITO_UPLOAD_PATH) !== false);
	            $excludeTransferitoFolder = (
		            !$selectedFoldersEnabled &&
		            $excludeTransferitoPlugin &&
		            strpos($filePath, $excludeTransferitoPlugin) !== false
	            );

	            if ($selectedFoldersEnabled && (count($folderPathArray) > 0)) {
		            $relativePath = $baseFolder . "/" . $relativePath;
	            }

                if ($relativePath !== "complete.zip" && !$excludeTransferitoFolder && !$excludeTransferitoUploadFolder) {
		            $archive->addFile($filePath, $relativePath);
                }

            }
        }
    }

    /**
     * @param \ZipArchive $archive
     * @param $filePath
     */
    public function createFileArchive($excludeTransferitoPlugin, \ZipArchive $archive, $filePath)
    {
        $fileArray = explode("/",$filePath);
        $fileName = $fileArray[count($fileArray) - 1];
        $archive->addFile($filePath, $fileName);
    }

    public function createArchive(
	    $excludeTransferitoPlugin,
    	$directoryToCopy,
	    $archiveLocation,
	    $archiveName,
	    $methodName,
	    $foldersEnabled = false,
	    $folderPaths = []
    )
    {
        $archivePath = $archiveLocation . "/" . $archiveName;
        $archivePublic = wp_upload_dir()["baseurl"] . "/transferito/" . $archiveName;
        $zip = new \ZipArchive();
        $zip->open($archivePath, \ZipArchive::CREATE | \ZipArchive::OVERWRITE);

	    if ($foldersEnabled && (count($folderPaths) > 0)) {
		    foreach ($folderPaths as $folder) {
			    $baseFolder = substr($folder, strlen($directoryToCopy));
			    call_user_func_array(
				    array($this, $methodName),
				    array(
					    $excludeTransferitoPlugin,
					    $zip,
					    "{$folder}/",
					    $baseFolder,
					    $foldersEnabled,
					    $folderPaths
				    )
			    );
		    }
	    } else {
		    call_user_func_array(
			    array($this, $methodName),
			    array(
				    $excludeTransferitoPlugin,
				    $zip,
				    $directoryToCopy
			    )
		    );
	    }

        $zip->close();

        /**
         * Set path in session
         */
        $transferitoPath = get_transient('transferito_file_paths');
        array_push($transferitoPath, $archivePath);
        set_transient('transferito_file_paths', $transferitoPath);

        return $archivePublic;
    }

    public function removeUsedArchives(array $archives)
    {
        foreach ($archives as $archive) {
            unlink($archive);
        }
    }

    /**
     * @param $archivePath
     */
    public function getArchiveFileSize($archivePath)
    {
        // @todo: Get archive size
    }

}