(function($) {

    $(document).ready(function(){

        var addedToQueue;
        var fileTransferResponse;

        function StatusCheck(token, successCallback) {

            this.token = token;
            this.poll = false;
            this.status = undefined;
            this.url = 'http://transferito.api/api/transfer/status/' + token;

            this.completed = successCallback;

            this.activatePoll = function () {
                this.poll = true;
                this.runPoll();
            };

            this.disablePoll = function () {
                this.poll = false;
            };

            this.setStatus = function(status) {
                this.status = status;
            };

            this.runPoll = function () {
                var self = this;
                var poll = setTimeout(function () {
                    $.ajax({
                        url: ajaxurl,
                        type: 'GET',
                        data: {
                            action: 'status_check',
                            token: self.token
                        },
                        success: function (response) {
                            if (response.data.completed) {
                                displayProgressRow('transferringFiles', true, 'Your files have been transferred to your new server');
                                self.disablePoll();
                                self.completed();
                            }
                        },
                        error: function(err) {
                            clearTimeout(poll);
                            self.disablePoll();
                        },
                        complete: function () {
                            if (self.poll === false) {
                                clearTimeout(poll);
                            } else {
                                self.runPoll();
                            }
                        }
                    })
                }, 3000);
            };
        }


        function convertFormArrayToObject(object, item) {
            if (item.name.indexOf('[]') !== -1) {
                var key = item.name.replace('[]', '');
                (!object[key]) ? object[key] = [item.value] : object[key].push(item.value);
            } else {
                object[item.name] = (item.value === 'on') ? true : item.value;
            }
            return object;
        }

        function displayResponse(message, noticeID, noticeMessage) {
            $('.transfer-inprogress p').each(function(index, val) {
                // $(val).hide();
            });
            if (message) {
                $('#' + noticeMessage).html(message);
            }
            $('#' + noticeID).show();
            $('#startTransfer').show();
            $('#transferitoToken').val('')
        }

        function displayProgressRow(mainID, success, message) {
            var theID = '#' + mainID + ' span.message';
            var theIconID = '#' + mainID + '_icon';

            var baseClass = 'dashicons-yes';
            if (!success) {
                baseClass = 'dashicons-no-alt';
            }
            var fullClass = 'dashicons ' + baseClass + ' transferito-' + success;

            $(theID).html(message);
            $(theIconID)
                .removeClass('transferito-loading')
                .addClass(fullClass);
        }

        function cleanUpFiles(filesSent, token, completion) {

            $('#cleaningUpFiles span.message').html('We\'re removing the backup we created');
            $('#cleaningUpFiles').show();

            var data = {
                action: 'clean_up_files'
            };
            var cleanup = $.post(ajaxurl, data, function(response) {});
            cleanup.complete(function(response) {
                var theResponse = response.responseJSON.data;
                var success = response.responseJSON.success;


                var noticeID = 'transferitoSuccess';
                var noticeMessage;
                var message;

                if (!addedToQueue) {
                    noticeID = 'transferitoError';
                    noticeMessage = 'transferitoErrorMessage';
                    message = fileTransferResponse;
                }

                displayProgressRow('cleaningUpFiles', success, theResponse.message);
                displayResponse(message, noticeID, noticeMessage);

                if (completion && response.status !== 200) {
                    completion.transferFailure();
                }

                if (response.status === 200 && filesSent) {
                    $('#transferringFiles span.message').html('We\'re copying your files to your new server');
                    $('#transferringFiles').show();
                    var statusCheck = new StatusCheck(token, completion.transferSuccess);
                    statusCheck.activatePoll();
                }

            });
        }

        function sendFiles(wpNonce, token, completion) {

            $('#sendingFiles span.message').html('We\'re transferring your files now');
            $('#sendingFiles').show();

            var data = {
                action: 'send_transfer_files',
                security: wpNonce
            };
            var sendFiles = $.post(ajaxurl, data, function(response) {});
            sendFiles.complete(function(response) {

                fileTransferResponse = (response.responseJSON.data.message)
                    ? response.responseJSON.data.message
                    : 'Error: we can\'t report, please try again shortly';
                addedToQueue = !!(response.responseJSON.success);

                /**
                 * Cleanup & change the icon
                 */
                displayProgressRow('sendingFiles', addedToQueue, fileTransferResponse);
                cleanUpFiles(addedToQueue, token, completion);

                if (completion && response.status !== 200) {
                    completion.transferFailure();
                }
            });
        }

        function startTransfer(token, completion) {

            $('#transferInitiated span.message').html('We\'re creating a backup of your files');
            $('#transferInitiated').show();

            var data = {
                action: 'preparing_transfer',
                security: $('#nonce').val(),
                token: token
            };
            var prepareTransfer = $.post(ajaxurl, data, function(response) {
                var theResponse = response.data;
                displayProgressRow('transferInitiated', response.success, theResponse.message);
                sendFiles(theResponse.nonce, token, completion);
            });
            prepareTransfer.fail(function(error) {
                var message = (error.responseJSON) ? error.responseJSON.data : 'There has been an issue preparing your files';
                displayResponse(message, 'transferitoError', 'transferitoErrorMessage');
            });
            prepareTransfer.complete(function(response) {
                if (completion && response.status !== 200) {
                    completion.transferFailure();
                }
            });
        }



        $('#fireTransfer').click(function() {

            var token = $('#transferitoToken').val();
            var errorMessage = 'You haven\'t provided a token';

            $('#transferitoError').hide();
            $('#transferitoSuccess').hide();
            $('#startTransfer').hide();
            $('#transferStartProgress').removeClass('transferito-hide');


            if (!token) {
                $('#transferStartProgress').addClass('transferito-hide');
                $('#startTransfer').show();
                displayResponse(errorMessage, 'transferitoError', 'transferitoErrorMessage');
            } else {
                startTransfer(token, {
                    transferSuccess: function() {
                        $('#transferProgressStartSpinner').addClass('transferito-hide');
                        $('#transferProgressHeader').html('Transfer complete');                        console.log('test success')
                    },
                    transferFailure: function() {
                        $('#transferStartProgress').addClass('transferito-hide');
                    }
                });
            }
        });

        /**
         *
         */
        $('.selected-folder_plus').click(function () {
            var childItems = $(this).next('.selected-folder_sub-directory');
            if (childItems.length === 0) {
                return false;
            }
            childItems.toggle();
        });

        $('.show-selected-folder').change(function() {
            $('.selected-folder-selection').toggle();
        });

        $('.toggle-saved_details').change(function() {
            $(this)
                .closest('tr')
                .next('.save-details-row')
                .toggle();
        });

        $('.hide-saved-detail').click(function() {
            $(this)
                .closest('.form-table').toggle()
                .next('.hide-detail-inputs')
                .toggle();
        });

        $('#excludeDatabase').change(function() {
           var databaseFields = $(this).closest('#databaseDetails').find('.database-fields');
           var checked = $(this).prop('checked');
           databaseFields.each(function(fieldIndex, field) {

               var fieldSelector = $(field);
               var fieldClasses = field.className.split(' ');
               var savedCheckbox = $('#saveDatabaseDetailsCheckbox');

               if (fieldClasses.length === 2 && fieldClasses.indexOf('save-details-row') !== -1) {
                   if (checked && savedCheckbox.prop('checked')) {
                        fieldSelector.hide();
                   } else if (!checked && savedCheckbox.prop('checked')) {
                       fieldSelector.show();
                   }
               } else {
                   fieldSelector.toggle();
               }
           });
        });

        $('#createTransfer').click(function() {

            $('#transferCreationForm').addClass('transferito-hide');
            $('#createTransferSuccess').removeClass('transferito-hide');
            $("html, body").animate({ scrollTop: 0 });

            var formElements = [];

            /**
             * Create array of form elements that aren't hidden
             * Include check for hidden form elements
             */
            $('.transferito-form-element').each(function(index, formElement) {
                var formSelector = $(formElement),
                    isVisible = formSelector.closest('table').css('display');

                if (isVisible === 'table' || isVisible === undefined) {
                    formElements.push(formElement);
                }
            });

            /**
             * Turn array of form elements
             * Into object with input names as properties & input values as value
             */
            var formData = $(formElements)
                .serializeArray()
                .reduce(function(object, item) {
                    return convertFormArrayToObject(object, item)
                }, {});

            /**
             * POST to create transfer endpoint
             */
            var createTransfer = $.post(ajaxurl,
                {
                    action: 'create_transfer',
                    security: $('#createTransferNonce').val(),
                    options: formData
                },
                function(response) {
                    var token = (response.data.token) ? response.data.token : '';
                    startTransfer(token, {
                        transferSuccess: function() {
                            $('#transferProgressStartSpinner').addClass('transferito-hide');
                            $('#transferProgressHeader').html('Transfer complete');
                        },
                        transferFailure: function() {
                            $('#transferProgressStartSpinner').addClass('transferito-hide');
                            $('#transferProgressHeader').html('Transfer failed');
                        }
                    });
                }
            );
            createTransfer.fail(function(error) {

                var errorMessage = (error.responseJSON) ? error.responseJSON.data : '';

                $('#transferCreationForm').removeClass('transferito-hide');
                $('#createTransferSuccess').addClass('transferito-hide');
                $('#transferCreationFailure').removeClass('transferito-hide');
                $('#failedTransferMessage').html(errorMessage);
            });

        });

    });


})(jQuery);





